      program driver
c     -----------------------------------------------------
c     Fortran driver for subroutine gqaps (sparse GRASP for
c     the QAP).
c     -----------------------------------------------------
c     Parameters, variables and arrays needed for the         
c     subroutine gqaps.
c     -----------------------------------------------------  
      integer nmax,nsq,in,out
      parameter (nmax=256,nsq=nmax*nmax,in=5,out=6)
      integer   f(nsq),d(nsq),a(nmax),b(nmax)
      integer   srtf(nsq),srtif(nsq),srtd(nsq)
      integer   srtid(nsq),srtc(nsq),srtic(nsq)
      integer   indexd(nsq),indexf(nsq),cost(nsq)
      integer   fdind(nsq),perm(nmax)
      integer   cp(nsq),optb(nmax)
      integer   iter,maxitr,bestv,n,look4,seed,n2
      real      alpha,beta,sparsd,sparsf
c     ------------------------------------------------------------------
c     Variables used by driver (not needed by gqaps).
c     ------------------------------------------------------------------
      integer   iseed0
c     ------------------------------------------------------------------
c     Read problem data and gqaps parameters.
c     Write out summary of input.
c
c     Read problem dimension (n).
c     ------------------------------------------------------------------
      read (in,*) n
      if (n .gt. nmax) then
           write(out,10)
10         format(' error: n > nmax  (increase nmax in driver.f)')
           stop
      else
           n2=n*n
c          -------------------------------------------------------------
c          Read matrices D and F and determine the sparsity of each.
c          -------------------------------------------------------------
           call readp(f,d,n,n2,sparsf,sparsd)
      endif
c     ------------------------------------------------------------------
c     Run sparse GRASP for QAP.
c     ------------------------------------------------------------------
c
c     Variables needed for input:
c       
c         n      : dimension of QAP
c                  integer
c         n2     : n * n
c         maxitr : maximum number of GRASP iterations
c                  integer
c         alpha  : GRASP construction phase parameter alpha (0.5)
c                  real  
c         beta   : GRASP construction phase parameter beta (0.1)
c                  real  
c         look4  : GRASP returns permutation if solution with cost
c                  less than or equal to look4 is found (look4 = -1
c                  causes GRASP to take maxitr iterations
c                  integer
c         seed   : seed for random number generator (270001)
c                  integer
c
c     Integer arrays needed for input:
c
c         f      : flow matrix (represented as row-by-row array of
c                  dimension NMAX*NMAX)
c         d      : distance matrix (represented as row-by-row array of
c                  dimension NMAX*NMAX)
c         
c     Integer arrays needed for work:
c   
c         a      : dimension NMAX
c         b      : dimension NMAX
c         optb   : dimension NMAX
c         srtf   : dimension NMAX*NMAX
c         srtif  : dimension NMAX*NMAX
c         srtd   : dimension NMAX*NMAX
c         srtid  : dimension NMAX*NMAX
c         srtc   : dimension NMAX*NMAX
c         srtic  : dimension NMAX*NMAX
c         indexd : dimension NMAX*NMAX
c         indexf : dimension NMAX*NMAX
c         cost   : dimension NMAX*NMAX
c         fdind  : dimension NMAX*NMAX
c         cp     : dimension NMAX*NMAX
c         
c     Integer array needed for output:
c   
c         perm   : permutation vector of best solution found 
c                  dimension NMAX
c
c     Integer variables needed for output:
c   
c         bestv  : cost of best assignment found
c         iter   : number of GRASP iterations taken  
c  
c     ------------------------------------------------------------------
c     Set GRASP parameters.
c     ------------------------------------------------------------------
      maxitr=1000
      alpha=.25
      beta=.5
      look4=-1
      seed=270001
      iseed0=seed

c     ------------------------------------------------------------------
c     Find approximate solution to QAP.
c     ------------------------------------------------------------------
      call gqaps(n,n2,maxitr,alpha,beta,look4,seed,f,d,a,b,
     &           optb,srtf,srtif,srtd,srtid,srtc,srtic,indexd,
     &           indexf,cost,fdind,perm,bestv,iter,cp)
c     ------------------------------------------------------------------
c     Write output summary.
c     ------------------------------------------------------------------
      call outsol(out,n,iter,iseed0,bestv,maxitr,look4,alpha,beta,perm,
     &            sparsf,sparsd)
c     ------------------------------------------------------------------
      stop
      end
